
-- ff_epicenter lua file.

IncludeScript("base_ctf");
IncludeScript("base_location");
IncludeScript("base_respawnturret");

-----------------------------------------------------------------------------
-- global overrides
-----------------------------------------------------------------------------
POINTS_PER_CAPTURE = 10;
FLAG_RETURN_TIME = 60
INITIAL_ROUND_LENGTH = 90

-----------------------------------------------------------------------------
-- startup 
-----------------------------------------------------------------------------

function startup()
	-- set up team limits
	local team = GetTeam( Team.kBlue )
	team:SetPlayerLimit( 0 )
	team:SetClassLimit( Player.kCivilian, -1 )

	team = GetTeam( Team.kRed )
	team:SetPlayerLimit( 0 )
	team:SetClassLimit( Player.kCivilian, -1 ) 

	team = GetTeam( Team.kYellow )
	team:SetPlayerLimit( -1 )	

	team = GetTeam( Team.kGreen )
	team:SetPlayerLimit( -1 )

	setup_door_timer("flag_wall", INITIAL_ROUND_LENGTH)

end

-----------------------------------------------------------------------------
-- flag stuff - flags need to be visually reversed
-----------------------------------------------------------------------------
blue_flag = baseflag:new({team = Team.kBlue,
						 modelskin = 1,
						 name = "Blue Flag",
						 hudicon = "hud_flag_red.vtf",
						 hudx = 5,
						 hudy = 162,
						 hudwidth = 48,
						 hudheight = 48,
						 hudalign = 1, 
						 hudstatusicondropped = "hud_flag_dropped_red.vtf",
						 hudstatusiconhome = "hud_flag_home_red.vtf",
						 hudstatusiconcarried = "hud_flag_carried_red.vtf",
						 hudstatusiconx = 60,
						 hudstatusicony = 5,
						 hudstatusiconw = 15,
						 hudstatusiconh = 15,
						 hudstatusiconalign = 3,
						 status = "home",
						 touchflags = {AllowFlags.kOnlyPlayers,AllowFlags.kRed,AllowFlags.kYellow,AllowFlags.kGreen}})

red_flag = baseflag:new({team = Team.kRed,
						 modelskin = 0,
						 name = "Red Flag",
						 hudicon = "hud_flag_blue.vtf",
						 hudx = 5,
					       hudy = 114,						 
						 hudwidth = 48,
						 hudheight = 48,
						 hudalign = 1,
						 hudstatusicondropped = "hud_flag_dropped_blue.vtf",
						 hudstatusiconhome = "hud_flag_home_blue.vtf",
						 hudstatusiconcarried = "hud_flag_carried_blue.vtf",
						 hudstatusiconx = 60,
						 hudstatusicony = 5,
						 hudstatusiconw = 15,
						 hudstatusiconh = 15,
						 hudstatusiconalign = 2,
						 status = "home",
						 touchflags = {AllowFlags.kOnlyPlayers,AllowFlags.kBlue,AllowFlags.kYellow,AllowFlags.kGreen}})
	
function baseflag:touch( touch_entity )
	local player = CastToPlayer( touch_entity )
	-- pickup if they can
	if self.notouch[player:GetId()] then return; end
	
	if player:GetTeamId() ~= self.team then

		-- if the player is a spy, then force him to lose his disguise
		player:SetDisguisable( false )
		-- if the player is a spy, then force him to lose his cloak
		player:SetCloakable( false )
		
		-- note: this seems a bit backwards (Pickup verb fits Player better)
		local flag = CastToInfoScript(entity)
		flag:Pickup(player)
		AddHudIcon( player, self.hudicon, flag:GetName(), self.hudx, self.hudy, self.hudwidth, self.hudheight, self.hudalign )
		RemoveHudItemFromAll( flag:GetName() .. "_h" )
		RemoveHudItemFromAll( flag:GetName() .. "_d" )
		AddHudIconToAll( self.hudstatusiconhome, ( flag:GetName() .. "_h" ), self.hudstatusiconx, self.hudstatusicony, self.hudstatusiconw, self.hudstatusiconh, self.hudstatusiconalign )
		AddHudIconToAll( self.hudstatusiconcarried, ( flag:GetName() .. "_c" ), self.hudstatusiconx, self.hudstatusicony, self.hudstatusiconw, self.hudstatusiconh, self.hudstatusiconalign )

	end
end

function baseflag:ondrop( owner_entity )
	local player = CastToPlayer( owner_entity )
	local flag = CastToInfoScript(entity)
	flag:EmitSound(self.tosssound)
end
-----------------------------------------------------------------------------
-- capture stuff - reset flags, players and return walls
-----------------------------------------------------------------------------
function setup_door_timer(doorname, duration)
	AddSchedule("round_start", duration, round_start, doorname)
	if duration > 65 then AddSchedule("round_60secwarn", duration-60, round_60secwarn) end
	if duration > 35 then AddSchedule("round_30secwarn", duration-30, round_30secwarn) end
	if duration > 15 then AddSchedule("round_10secwarn", duration-10, round_10secwarn) end
end

function reset_door(doorname)
	CloseDoor(doorname)
end

function round_start(doorname)
	BroadCastMessage("Flags are now available!")
	OpenDoor(doorname)
end

function round_60secwarn() BroadCastMessage("60 Seconds until round starts") end
function round_30secwarn() BroadCastMessage("30 Seconds until round starts") end
function round_10secwarn() BroadCastMessage("10 Seconds until round starts") end

function basecap:oncapture(player, item)
	FLAG_RETURN_TIME = 1;
	-- let the teams know that a capture occured
	SmartSound(player, "yourteam.flagcap", "yourteam.flagcap", "otherteam.flagcap")
	SmartSound(player, "vox.yourcap", "vox.yourcap", "vox.enemycap")
	SmartMessage(player, "#FF_YOUCAP", "#FF_TEAMCAP", "#FF_OTHERTEAMCAP")

	reset_door("flag_wall")
	ApplyToAll({ AT.kReturnCarriedItems, AT.kReturnDroppedItems, AT.kRemovePacks, AT.kRemoveProjectiles, AT.kRemoveBuildables, AT.kRemoveRagdolls, AT.kStopPrimedGrens, AT.kReloadClips, AT.kRespawnPlayers})
	setup_door_timer("flag_wall", 65)
	FLAG_RETURN_TIME = 45;
	
end





-----------------------------------------------------------------------------
-- Locations
-----------------------------------------------------------------------------
location_bluespawn = location_info:new({ text = "Blue Spawn", team = Team.kBlue })
location_bluecap = location_info:new({ text = "Blue Cap Area", team = Team.kBlue })
location_blueyard = location_info:new({ text = "Blue Yard", team = Team.kBlue })
location_bluestreets = location_info:new({ text = "Blue Streets", team = Team.kBlue })
location_bluebalc = location_info:new({ text = "Blue Balcony", team = Team.kBlue })
location_vluebuild = location_info:new({ text = "Blue Building", team = Team.kBlue })

location_redspawn = location_info:new({ text = "Red Spawn", team = Team.kRed })
location_redcap = location_info:new({ text = "Red Cap Area", team = Team.kRed })
location_redyard = location_info:new({ text = "Red Yard", team = Team.kRed })
location_redstreets = location_info:new({ text = "Red Streets", team = Team.kRed })
location_redbalc = location_info:new({ text = "Red Balcony", team = Team.kRed })
location_redbuild = location_info:new({ text = "Red Building", team = Team.kRed })

location_bridge = location_info:new({ text = "The Bridge", team = NO_TEAM })