
-- ff_monkey.lua

-----------------------------------------------------------------------------
-- includes
-----------------------------------------------------------------------------
IncludeScript("base_ctf")
IncludeScript("base_location")
IncludeScript("base_respawnturret")

-----------------------------------------------------------------------------
-- global overrides
-----------------------------------------------------------------------------
POINTS_PER_CAPTURE = 10
FLAG_RETURN_TIME = 60

function startup()
	-- set up team limits on each team
	SetPlayerLimit(Team.kBlue, -1)
	SetPlayerLimit(Team.kRed, 0)
	SetPlayerLimit(Team.kYellow, -1)
	SetPlayerLimit(Team.kGreen, 0)

	-- CTF maps generally don't have civilians,
	-- so override in map LUA file if you want 'em
	local team = GetTeam(Team.kGreen)
	team:SetClassLimit(Player.kCivilian, -1)

	team = GetTeam(Team.kRed)
	team:SetClassLimit(Player.kCivilian, -1)
end

function flaginfo( player_entity )
-- at the moment this is crappy workaround - simply displays the home icon
	local player = CastToPlayer( player_entity )

	-- copied from green_flag variables
	AddHudIcon( player, green_flag.hudstatusiconhome, ( green_flag.name.. "_h" ), green_flag.hudstatusiconx, green_flag.hudstatusicony, green_flag.hudstatusiconw, green_flag.hudstatusiconh, green_flag.hudstatusiconalign )
	AddHudIcon( player, red_flag.hudstatusiconhome, ( red_flag.name.. "_h" ), red_flag.hudstatusiconx, red_flag.hudstatusicony, red_flag.hudstatusiconw, red_flag.hudstatusiconh, red_flag.hudstatusiconalign )
	
	local flag = GetInfoScriptByName("green_flag")
	
	if flag:IsCarried() then
			AddHudIcon( player, green_flag.hudstatusiconcarried, ( green_flag.name.. "_h" ), green_flag.hudstatusiconx, green_flag.hudstatusicony, green_flag.hudstatusiconw, green_flag.hudstatusiconh, green_flag.hudstatusiconalign )
	elseif flag:IsDropped() then
			AddHudIcon( player, green_flag.hudstatusicondropped, ( green_flag.name.. "_h" ), green_flag.hudstatusiconx, green_flag.hudstatusicony, green_flag.hudstatusiconw, green_flag.hudstatusiconh, green_flag.hudstatusiconalign )
	end

	flag = GetInfoScriptByName("red_flag")
	
	if flag:IsCarried() then
			AddHudIcon( player, red_flag.hudstatusiconcarried, ( red_flag.name.. "_h" ), red_flag.hudstatusiconx, red_flag.hudstatusicony, red_flag.hudstatusiconw, red_flag.hudstatusiconh, red_flag.hudstatusiconalign )
	elseif flag:IsDropped() then
			AddHudIcon( player, red_flag.hudstatusicondropped, ( red_flag.name.. "_h" ), red_flag.hudstatusiconx, red_flag.hudstatusicony, red_flag.hudstatusiconw, red_flag.hudstatusiconh, red_flag.hudstatusiconalign )
	end
end

-----------------------------------------------------------------------------
-- Pickups
-----------------------------------------------------------------------------

monkeypackgeneric = genericbackpack:new({
	health = 20,
	armor = 15,
	grenades = 60,
	nails = 60,
	shells = 60,
	rockets = 60,
	cells = 60,
	gren1 = 1,
	gren2 = 1,
	respawntime = 35,
	model = "models/items/backpack/backpack.mdl",
	materializesound = "Item.Materialize",
	touchsound = "Backpack.Touch",
	botgoaltype = Bot.kBackPack_Ammo
})

function monkeypackgeneric:dropatspawn() return false end

redmonkeypack = genericbackpack:new({
	health = 200,
	armor = 200,
	grenades = 90,
	nails = 90,
	shells = 90,
	rockets = 90,
	cells = 90,
	model = "models/items/backpack/backpack.mdl",
	materializesound = "Item.Materialize",
	touchsound = "Backpack.Touch",
	touchflags = {AllowFlags.kRed},
	botgoaltype = Bot.kBackPack_Ammo
})

function redmonkeypack:dropatspawn() return false end

redmonkeypacktoo = genericbackpack:new({
	health = 20,
	armor = 15,
	grenades = 10,
	nails = 30,
	shells = 30,
	rockets = 10,
	cells = 30,
	respawntime = 20,
	model = "models/items/backpack/backpack.mdl",
	materializesound = "Item.Materialize",
	touchsound = "Backpack.Touch",
	touchflags = {AllowFlags.kRed},
	botgoaltype = Bot.kBackPack_Ammo
})

function redmonkeypacktoo:dropatspawn() return false end

greenmonkeypack = genericbackpack:new({
	health = 200,
	armor = 200,
	grenades = 90,
	nails = 90,
	shells = 90,
	rockets = 90,
	cells = 90,
	model = "models/items/backpack/backpack.mdl",
	materializesound = "Item.Materialize",
	touchsound = "Backpack.Touch",
	touchflags = {AllowFlags.kGreen},
	botgoaltype = Bot.kBackPack_Ammo
})

function greenmonkeypack:dropatspawn() return false end

greenmonkeypacktoo = genericbackpack:new({
	health = 20,
	armor = 15,
	grenades = 10,
	nails = 30,
	shells = 30,
	rockets = 10,
	cells = 30,
	respawntime = 20,
	model = "models/items/backpack/backpack.mdl",
	materializesound = "Item.Materialize",
	touchsound = "Backpack.Touch",
	touchflags = {AllowFlags.kGreen},
	botgoaltype = Bot.kBackPack_Ammo
})

function greenmonkeypacktoo:dropatspawn() return false end

redmonkeygrenades = genericbackpack:new({
	detpacks = 1,
	gren1 = 4,
	gren2 = 4,
	model = "models/items/backpack/backpack.mdl",
	materializesound = "Item.Materialize",
	touchsound = "Backpack.Touch",
	touchflags = {AllowFlags.kRed},
	botgoaltype = Bot.kBackPack_Ammo
})

function redmonkeygrenades:dropatspawn() return false end

greenmonkeygrenades = genericbackpack:new({
	detpacks = 1,
	gren1 = 4,
	gren2 = 4,
	model = "models/items/backpack/backpack.mdl",
	materializesound = "Item.Materialize",
	touchsound = "Backpack.Touch",
	touchflags = {AllowFlags.kGreen},
	botgoaltype = Bot.kBackPack_Ammo
})

function greenmonkeygrenades:dropatspawn() return false end
-----------------------------------------------------------------------------
-- Locations
-----------------------------------------------------------------------------
location_green_frupper = location_info:new({ text = "Flag Room Catwalks", team = Team.kGreen })
location_red_frupper = location_info:new({ text = "Flag Room Catwalks", team = Team.kRed })

location_green_frlower = location_info:new({ text = "Lower Flag Room", team = Team.kGreen })
location_red_frlower = location_info:new({ text = "Lower Flag Room", team = Team.kRed })

location_green_frwater = location_info:new({ text = "Flag Room Water", team = Team.kGreen })
location_red_frwater = location_info:new({ text = "Flag Room Water", team = Team.kRed })

location_green_ladder = location_info:new({ text = "Battlements Ladder", team = Team.kGreen })
location_red_ladder = location_info:new({ text = "Battlements Ladder", team = Team.kRed })

location_green_concexit = location_info:new({ text = "Conc Route Exit", team = Team.kGreen })
location_red_concexit = location_info:new({ text = "Conc Route Exit", team = Team.kRed })

location_green_canal = location_info:new({ text = "Yard Canal", team = Team.kGreen })
location_red_canal = location_info:new({ text = "Yard Canal", team = Team.kRed })


-----------------------------------------------------------------------------
-- Doors
-----------------------------------------------------------------------------

